@@ -22,12 +22,17 @@ class DataflowJobSubmitter(AbstractJobSubmitter):
         if o.scheme != 'oci':
             raise SparkETLLaunchFailure("run_base_dir must be in OCI")
 
-    
+
     @property
     def region(self):
         return self.config['region']
 
     def run(self, deployment_location, options={}, args={}):
+        # options fields
+        #     num_executors     : number
+        #     driver_shape      :  string
+        #     executor_shape    : string
+        #     lib_url_duration  : number (repre the number of minutes)
         o = urlparse(deployment_location)
         if o.scheme != 'oci':
             raise SparkETLLaunchFailure("deployment_location must be in OCI")
@@ -40,9 +45,10 @@ class DataflowJobSubmitter(AbstractJobSubmitter):
         root_path = o.path[1:] # remove the leading "/"
 
         # let's get the deployment.json
-        os_client = get_os_client(self.region)
+        os_client = get_os_client(self.region, self.config.get("oci_config"))
         deployment = os_download_json(os_client, namespace, bucket, f"{root_path}/deployment.json")
 
+        lib_url_duration = options.get("lib_url_duration", 30)
         r = os_client.create_preauthenticated_request(
             namespace,
             bucket,
@@ -50,7 +56,7 @@ class DataflowJobSubmitter(AbstractJobSubmitter):
                 access_type = 'ObjectRead',
                 name=f'for run {run_id}',
                 object_name=f'{root_path}/lib.zip',
-                time_expires=datetime.utcnow() + timedelta(minutes=30)
+                time_expires=datetime.utcnow() + timedelta(minutes=lib_url_duration)
             )
         )
         check_response(r, lambda : SparkETLLaunchFailure("dataflow failed to get lib url"))
@@ -63,7 +69,7 @@ class DataflowJobSubmitter(AbstractJobSubmitter):
         root_path = o.path[1:] # remove the leading "/"
         os_upload_json(os_client, args, namespace, bucket, f"{root_path}/{run_id}/args.json")
 
-        df_client = get_df_client(self.region)
+        df_client = get_df_client(self.region, self.config.get("oci_config"))
         crd_argv = {
             'compartment_id': deployment['compartment_id'],
             'application_id': deployment['application_id'],
@@ -84,8 +90,8 @@ class DataflowJobSubmitter(AbstractJobSubmitter):
         r = df_client.create_run(create_run_details=create_run_details)
         check_response(r, lambda : SparkETLLaunchFailure("dataflow failed to run the application"))
         run = r.data
-        run_id = run.id
-        print(f"Job launched, run_id = {run_id}")
+        oci_run_id = run.id
+        print(f"Job launched, run_id = {run_id}, oci_run_id = {run.id}")
 
         while True:
             time.sleep(10)
@@ -93,14 +99,17 @@ class DataflowJobSubmitter(AbstractJobSubmitter):
             check_response(r, lambda : SparkETLGetStatusFailure("dataflow failed to get run status"))
             run = r.data
             print(f"Status: {run.lifecycle_state}")
-            if run.lifecycle_state in ('FAILED', 'SUCCEEDED'):
+            if run.lifecycle_state in ('FAILED', 'SUCCEEDED', 'CANCELED'):
                 break
-        
-        return {
-            'state': run.lifecycle_state,
-            'run_id': run_id,
-            'succeeded': run.lifecycle_state == 'SUCCEEDED'
-        }
+
+        if run.lifecycle_state in ('FAILED', 'CANCELED'):
+            raise Exception(f"Job failed with status: {run.lifecycle_state}")
+        return self.get_result(run_id)
+        # return {
+        #     'state': run.lifecycle_state,
+        #     'run_id': run_id,
+        #     'succeeded': run.lifecycle_state == 'SUCCEEDED'
+        # }
 
     def get_result(self, run_id):
         result_object_name = f"{self.config['run_base_dir']}/{run_id}/result.json"
@@ -110,7 +119,7 @@ class DataflowJobSubmitter(AbstractJobSubmitter):
         bucket = o.netloc.split('@')[0]
         object_name = o.path[1:]    # remove the leading "/"
 
-        os_client = get_os_client(self.region)
+        os_client = get_os_client(self.region, self.config.get("oci_config"))
         result = os_download_json(os_client, namespace, bucket, object_name)
         return result
 
